require "Apollo"
require "ApolloColor" 
require "XmlDoc"
  
local AlpacaWindowLibrary = {}

AlpacaWindowLibrary.EnumMarkerType = {
  Arrow = 1,
  Line = 2
}

AlpacaWindowLibrary.EnumPosition = {
  Left = 1,
  Right = 2,
  Top = 3,
  Bottom = 4
}

AlpacaWindowLibrary.EnumDirection = {
  Left = 1,
  Right = 2,
  Up = 3,
  Down = 4
}

function AlpacaWindowLibrary:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self 
  return o
end

function AlpacaWindowLibrary:Init()
  self.bIsLoaded = false
  
  Apollo.RegisterPackage(self, "AlpacaWindowLibrary", 1, {})
end

function AlpacaWindowLibrary:OnLoad()
  Apollo.LoadSprites("AlpacaSprites.xml")

  self.xmlDoc = XmlDoc.CreateFromFile("AlpacaWindowLibrary.xml")
  self.xmlDoc:RegisterCallback("OnDocLoaded", self)  
end

function AlpacaWindowLibrary:OnDocLoaded()
  if self.xmlDoc == nil or not self.xmlDoc:IsLoaded() then return end

  self.bIsLoaded = true
end

function AlpacaWindowLibrary:IsLoaded()
    return self.bIsLoaded
end

function AlpacaWindowLibrary:AlpacaColor(fRed, fGreen, fBlue, fAlpha)
  return { fRed = fRed, fGreen = fGreen, fBlue = fBlue, fAlpha = fAlpha } 
end

function AlpacaWindowLibrary:ToApolloColor(crColor)
  return ApolloColor.new(crColor.fRed, crColor.fGreen, crColor.fBlue, crColor.fAlpha)  
end

function AlpacaWindowLibrary:ToAlpacaColor(crColor)
  local tColor = crColor:ToTable()
  
  return { fRed = tColor.r, fGreen = tColor.g, fBlue = tColor.b, fAlpha = tColor.a }
end

function AlpacaWindowLibrary:FindParent(wndFrame, strName)
  local wndParent = wndFrame:GetParent()
  
  if not wndParent or wndParent:GetName() == strName then
    return wndParent
  else
    return self:FindParent(wndParent, strName)
  end
end

function AlpacaWindowLibrary:CreateBar(tBarSettings, wndParent)
  local wndBar = Apollo.LoadForm(self.xmlDoc, "Bar", wndParent, self)
  
  local nHorizontal = tBarSettings.tPosition and tBarSettings.tPosition.nHorizontal or 0
  local nVertical = tBarSettings.tPosition and tBarSettings.tPosition.nVertical or 0
  local nWidth = tBarSettings.tSize and tBarSettings.tSize.nWidth or 0
  local nHeight = tBarSettings.tSize and tBarSettings.tSize.nHeight or 0 
  
  local nBarOffsetLeft = nHorizontal - math.ceil(nWidth / 2)
  local nBarOffsetTop = nVertical - math.ceil(nHeight / 2)
  local nBarOffsetRight = nBarOffsetLeft + nWidth
  local nBarOffsetBottom = nBarOffsetTop + nHeight
  
  wndBar:SetAnchorOffsets(nBarOffsetLeft, nBarOffsetTop, nBarOffsetRight, nBarOffsetBottom)
  
  local wndDuration = wndBar:FindChild("Duration")
  local wndIcon = wndBar:FindChild("Icon")
  local nIconSize = tBarSettings.nIconSize or 0
  local eIconPosition = tBarSettings.eIconPosition or self.EnumPosition.Bottom
    
  if eIconPosition == self.EnumPosition.Left then
    wndIcon:SetAnchorPoints(0, 0.5, 0, 0.5)
    wndIcon:SetAnchorOffsets(0, -1 * math.ceil(nIconSize / 2), nIconSize, math.floor(nIconSize / 2))
    wndDuration:SetAnchorOffsets(nIconSize, 0, 0, 0)
  elseif eIconPosition == self.EnumPosition.Right then
    wndIcon:SetAnchorPoints(1, 0.5, 1, 0.5)
    wndIcon:SetAnchorOffsets(-1 * nIconSize, -1 * math.ceil(nIconSize / 2), 0, math.floor(nIconSize / 2))
    wndDuration:SetAnchorOffsets(0, 0, -1 * nIconSize, 0)
  elseif eIconPosition == self.EnumPosition.Top then
    wndIcon:SetAnchorPoints(0.5, 0, 0.5, 0)
    wndIcon:SetAnchorOffsets(-1 * math.ceil(nIconSize / 2), 0, math.floor(nIconSize / 2), nIconSize)
    wndDuration:SetAnchorOffsets(0, nIconSize, 0, 0)
  else
    wndIcon:SetAnchorPoints(0.5, 1, 0.5, 1)
    wndIcon:SetAnchorOffsets(-1 * math.ceil(nIconSize / 2), -1 * nIconSize, math.floor(nIconSize / 2), 0)
    wndDuration:SetAnchorOffsets(0, 0, 0, -1 * nIconSize)
  end
  
  if nIconSize == 0 then
    wndIcon:Destroy()
  end
  
  local wndBorder = wndBar:FindChild("Border")
  local crBorder = tBarSettings.crBorder and self:ToApolloColor(tBarSettings.crBorder) or ApolloColor.new(1, 1, 1, 1)
  wndBorder:SetBGColor(crBorder)
  
  local wndFill = wndBar:FindChild("Fill")
  local crFill = tBarSettings.crFill and self:ToApolloColor(tBarSettings.crFill) or ApolloColor.new(1, 1, 1, 1)
  wndFill:SetBGColor(crFill)
  
  self:UpdateBarFill(wndBar, tBarSettings, 0)

  return wndBar  
end

function AlpacaWindowLibrary:UpdateBarFill(wndBar, tBarSettings, fPercent)
  local wndFill = wndBar:FindChild("Fill")
  local eDirection = tBarSettings.eFillDirection
  
  if eDirection == self.EnumDirection.Right then  
    wndFill:SetAnchorPoints(0, 0, fPercent, 1)
  elseif eDirection == self.EnumDirection.Left then
    wndFill:SetAnchorPoints(1 - fPercent, 0, 1, 1)
  elseif eDirection == self.EnumDirection.Down then
    wndFill:SetAnchorPoints(0, 0, 1, fPercent)
  else
    wndFill:SetAnchorPoints(0, 1 - fPercent, 1, 1)
  end
end

function AlpacaWindowLibrary:CreateMarker(eMarkerType, tBarSettings, wndParent, fPercent)
  local wndMarker = Apollo.LoadForm(self.xmlDoc, "EmptyFrame", wndParent, self)
  wndMarker:SetName("Marker")
  
  local eDirection = tBarSettings.eFillDirection
  local crColor = tBarSettings.tMarker.crColor and self:ToApolloColor(tBarSettings.tMarker.crColor) or ApolloColor.new(1, 1, 1, 1)
  local nWidth = tBarSettings.tMarker.nWidth
  local nHeight = tBarSettings.tMarker.nHeight

  if eMarkerType == self.EnumMarkerType.Arrow then    
    local wndArrow1 = Apollo.LoadForm(self.xmlDoc, "EmptyFrame", wndMarker, self)
    local wndArrow2 = Apollo.LoadForm(self.xmlDoc, "EmptyFrame", wndMarker, self)
    
    wndArrow1:SetBGColor(crColor)
    wndArrow2:SetBGColor(crColor)
    
    if eDirection == self.EnumDirection.Left or eDirection == self.EnumDirection.Right then
      local fArrowLength = (nHeight - wndParent:GetHeight()) / 2

      wndArrow1:SetSprite("AlpacaSprites:ArrowDown")
      wndArrow1:SetAnchorPoints(0, 0, 1, 0)
      wndArrow1:SetAnchorOffsets(0, 0, 0, math.floor(fArrowLength))
      
      wndArrow2:SetSprite("AlpacaSprites:ArrowUp")
      wndArrow2:SetAnchorPoints(0, 1, 1, 1)
      wndArrow2:SetAnchorOffsets(0, -1 * math.floor(fArrowLength), 0, 0)
    else
      local fArrowLength = (nWidth - wndParent:GetWidth()) / 2

      wndArrow1:SetSprite("AlpacaSprites:ArrowRight")
      wndArrow1:SetAnchorPoints(0, 0, 0, 1)
      wndArrow1:SetAnchorOffsets(0, 0, math.floor(fArrowLength), 0)
      
      wndArrow2:SetSprite("AlpacaSprites:ArrowLeft")
      wndArrow2:SetAnchorPoints(1, 0, 1, 1)
      wndArrow2:SetAnchorOffsets(-1 * math.floor(fArrowLength), 0, 0, 0) 
    end
  else
    wndMarker:SetSprite("AlpacaSpites:WhiteFill")
    wndMarker:SetBGColor(crColor)
  end
    
  self:MoveMarker(wndMarker, tBarSettings, fPercent)
    
  wndMarker:SetAnchorOffsets(-1 * math.ceil(nWidth / 2), -1 * math.ceil(nHeight / 2), math.floor(nWidth / 2), math.floor(nHeight / 2))
  
  return wndMarker
end

function AlpacaWindowLibrary:MoveMarker(wndMarker, tBarSettings, fPercent)
  local eDirection = tBarSettings.eFillDirection
  
  if eDirection == self.EnumDirection.Right then  
    wndMarker:SetAnchorPoints(fPercent, 0.5, fPercent, 0.5)
  elseif eDirection == self.EnumDirection.Left then
    wndMarker:SetAnchorPoints(1 - fPercent, 0.5, 1 - fPercent, 0.5)
  elseif eDirection == self.EnumDirection.Down then
    wndMarker:SetAnchorPoints(0.5, fPercent, 0.5, fPercent)
  else
    wndMarker:SetAnchorPoints(0.5, 1 - fPercent, 0.5, 1 - fPercent)
  end
end

function AlpacaWindowLibrary:CreateEmptyFrame(wndParent)
  return Apollo.LoadForm(self.xmlDoc, "EmptyFrame", wndParent, self)
end

local AlpacaWindowLibraryInst = AlpacaWindowLibrary:new()
AlpacaWindowLibraryInst:Init()